package com.link.core.mq.cmq.Tools;

import com.link.core.mq.cmq.Tools.Json.JSONArray;
import com.link.core.mq.cmq.Tools.Json.JSONObject;
import java.util.List;
import java.util.TreeMap;
import java.util.Vector;

public class Topic {
   protected String topicName;
   protected CMQClient client;

   Topic(String topicName, CMQClient client) {
      this.topicName = topicName;
      this.client = client;
   }

   public TopicMeta getTopicAttributes() throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("topicName", this.topicName);
      String result = this.client.call("GetTopicAttributes", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         TopicMeta meta = new TopicMeta();
         meta.setMsgCount(jsonObj.getInt("msgCount"));
         meta.setMaxMsgSize(jsonObj.getInt("maxMsgSize"));
         meta.setMsgRetentionSeconds(jsonObj.getInt("msgRetentionSeconds"));
         meta.setCreateTime(jsonObj.getInt("createTime"));
         meta.setLastModifyTime(jsonObj.getInt("lastModifyTime"));
         meta.setFilterType(jsonObj.getInt("filterType"));
         return meta;
      }
   }

   public void setTopicAttributes(final int maxMsgSize) throws Exception {
      if (maxMsgSize >= 0 && maxMsgSize <= 65536) {
         TreeMap<String, String> param = new TreeMap();
         param.put("topicName", this.topicName);
         if (maxMsgSize > 0) {
            param.put("maxMsgSize", Integer.toString(maxMsgSize));
         }

         String result = this.client.call("SetTopicAttributes", param);
         JSONObject jsonObj = new JSONObject(result);
         int code = jsonObj.getInt("code");
         if (code != 0) {
            throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
         }
      } else {
         throw new CMQClientException("Invalid parameter maxMsgSize < 0 or maxMsgSize > 65536");
      }
   }

   public String publishMessage(String message) throws Exception {
      return this.publishMessage(message, (List)null, (String)null);
   }

   public String publishMessage(String message, String routingKey) throws Exception {
      return this.publishMessage(message, (List)null, routingKey);
   }

   public String publishMessage(String msg, List<String> vTagList, String routingKey) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("topicName", this.topicName);
      param.put("msgBody", msg);
      if (routingKey != null) {
         param.put("routingKey", routingKey);
      }

      if (vTagList != null) {
         for(int i = 0; i < vTagList.size(); ++i) {
            param.put("msgTag." + Integer.toString(i + 1), vTagList.get(i));
         }
      }

      String result = this.client.call("PublishMessage", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         return jsonObj.getString("msgId");
      }
   }

   public Vector<String> batchPublishMessage(List<String> vMsgList) throws Exception {
      return this.batchPublishMessage(vMsgList, (List)null, (String)null);
   }

   public Vector<String> batchPublishMessage(List<String> vMsgList, String routingKey) throws Exception {
      return this.batchPublishMessage(vMsgList, (List)null, routingKey);
   }

   public Vector<String> batchPublishMessage(List<String> vMsgList, List<String> vTagList, String routingKey) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("topicName", this.topicName);
      if (routingKey != null) {
         param.put("routingKey", routingKey);
      }

      int i;
      if (vMsgList != null) {
         for(i = 0; i < vMsgList.size(); ++i) {
            param.put("msgBody." + Integer.toString(i + 1), vMsgList.get(i));
         }
      }

      if (vTagList != null) {
         for(i = 0; i < vTagList.size(); ++i) {
            param.put("msgTag." + Integer.toString(i + 1), vTagList.get(i));
         }
      }

      String result = this.client.call("BatchPublishMessage", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         JSONArray jsonArray = jsonObj.getJSONArray("msgList");
         Vector<String> vmsgId = new Vector();

         for(int i = 0; i < jsonArray.length(); ++i) {
            JSONObject obj = (JSONObject)jsonArray.get(i);
            vmsgId.add(obj.getString("msgId"));
         }

         return vmsgId;
      }
   }

   public int ListSubscription(final int offset, int limit, final String searchWord, List<String> vSubscriptionList) throws Exception {
      TreeMap<String, String> param = new TreeMap();
      param.put("topicName", this.topicName);
      if (searchWord != null && !"".equals(searchWord)) {
         param.put("searchWord", searchWord);
      }

      if (offset >= 0) {
         param.put("offset", Integer.toString(offset));
      }

      if (limit > 0) {
         param.put("limit", Integer.toString(limit));
      }

      String result = this.client.call("ListSubscriptionByTopic", param);
      JSONObject jsonObj = new JSONObject(result);
      int code = jsonObj.getInt("code");
      if (code != 0) {
         throw new CMQServerException(code, jsonObj.getString("message"), jsonObj.getString("requestId"));
      } else {
         int totalCount = jsonObj.getInt("totalCount");
         JSONArray jsonArray = jsonObj.getJSONArray("subscriptionList");

         for(int i = 0; i < jsonArray.length(); ++i) {
            JSONObject obj = (JSONObject)jsonArray.get(i);
            vSubscriptionList.add(obj.getString("subscriptionName"));
         }

         return totalCount;
      }
   }
}
